__author__ = 'Aaron Mussig'
__author_email__ = 'aaronmussig@gmail.com'
__copyright__ = 'Copyright 2023'
__credits__ = ['Aaron Mussig']
__description__ = 'Automatic pre-curation of GTDB trees.'
__email__ = 'aaronmussig@gmail.com'
__license__ = 'GPL3'
__maintainer__ = 'Aaron Mussig'
__maintainer_email__ = 'aaronmussig@gmail.com'
__name__ = 'gtdb_precurate'
__python_requires__ = '>=3.8'
__status__ = 'Production'
__title__ = 'GTDB Precurate'
__url__ = 'https://github.com/Ecogenomics/gtdb_precurate'
__version__ = '1.0.0'
